"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventClient = exports.LogLevel = void 0;
const events_1 = require("../../protoc/events/events");
const hatchet_error_1 = __importDefault(require("../../util/errors/hatchet-error"));
const retrier_1 = require("../../util/retrier");
// eslint-disable-next-line no-shadow
var LogLevel;
(function (LogLevel) {
    LogLevel["INFO"] = "INFO";
    LogLevel["WARN"] = "WARN";
    LogLevel["ERROR"] = "ERROR";
    LogLevel["DEBUG"] = "DEBUG";
})(LogLevel || (exports.LogLevel = LogLevel = {}));
class EventClient {
    constructor(config, channel, factory) {
        this.config = config;
        this.client = factory.create(events_1.EventsServiceDefinition, channel);
        this.logger = config.logger(`Dispatcher`, config.log_level);
        this.retrier = retrier_1.retrier;
    }
    push(type, input, options = {}) {
        var _a;
        const namespacedType = `${(_a = this.config.namespace) !== null && _a !== void 0 ? _a : ''}${type}`;
        const req = {
            key: namespacedType,
            payload: JSON.stringify(input),
            eventTimestamp: new Date(),
            additionalMetadata: options.additionalMetadata
                ? JSON.stringify(options.additionalMetadata)
                : undefined,
        };
        try {
            const e = this.retrier(() => __awaiter(this, void 0, void 0, function* () { return this.client.push(req); }), this.logger);
            this.logger.info(`Event pushed: ${namespacedType}`);
            return e;
        }
        catch (e) {
            throw new hatchet_error_1.default(e.message);
        }
    }
    bulkPush(type, inputs, options = {}) {
        var _a;
        const namespacedType = `${(_a = this.config.namespace) !== null && _a !== void 0 ? _a : ''}${type}`;
        const events = inputs.map((input) => {
            return {
                key: namespacedType,
                payload: JSON.stringify(input.payload),
                eventTimestamp: new Date(),
                additionalMetadata: (() => {
                    if (input.additionalMetadata) {
                        return JSON.stringify(input.additionalMetadata);
                    }
                    if (options.additionalMetadata) {
                        return JSON.stringify(options.additionalMetadata);
                    }
                    return undefined;
                })(),
            };
        });
        const req = {
            events,
        };
        try {
            const res = this.retrier(() => __awaiter(this, void 0, void 0, function* () { return this.client.bulkPush(req); }), this.logger);
            this.logger.info(`Bulk events pushed for type: ${namespacedType}`);
            return res;
        }
        catch (e) {
            throw new hatchet_error_1.default(e.message);
        }
    }
    putLog(stepRunId, log, level) {
        const createdAt = new Date();
        if (log.length > 1000) {
            this.logger.warn(`log is too long, skipping: ${log.length} characters`);
            return;
        }
        //  fire and forget the log
        this.client
            .putLog({
            stepRunId,
            createdAt,
            message: log,
            level: level || LogLevel.INFO,
        })
            .catch((e) => {
            // log a warning, but this is not a fatal error
            this.logger.warn(`Could not put log: ${e.message}`);
        });
    }
    putStream(stepRunId, data) {
        const createdAt = new Date();
        let dataBytes;
        if (typeof data === 'string') {
            dataBytes = new TextEncoder().encode(data);
        }
        else if (data instanceof Uint8Array) {
            dataBytes = data;
        }
        else {
            throw new Error('Invalid data type. Expected string or Uint8Array.');
        }
        (0, retrier_1.retrier)(() => __awaiter(this, void 0, void 0, function* () {
            return this.client.putStreamEvent({
                stepRunId,
                createdAt,
                message: dataBytes,
            });
        }), this.logger).catch((e) => {
            // log a warning, but this is not a fatal error
            this.logger.warn(`Could not put log: ${e.message}`);
        });
    }
}
exports.EventClient = EventClient;
