import { AdminClient } from '../../admin';
import { Api } from '../../rest';
import { CronWorkflows, CronWorkflowsList } from '../../rest/generated/data-contracts';
import { z } from 'zod';
import { Workflow } from '../../../workflow';
import { ClientConfig } from '../client-config';
/**
 * Schema for creating a Cron Trigger.
 */
export declare const CreateCronTriggerSchema: z.ZodObject<{
    name: z.ZodString;
    expression: z.ZodEffects<z.ZodString, string, string>;
    input: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodAny>>;
    additionalMetadata: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
}, "strip", z.ZodTypeAny, {
    name: string;
    expression: string;
    additionalMetadata?: Record<string, string> | undefined;
    input?: Record<string, any> | undefined;
}, {
    name: string;
    expression: string;
    additionalMetadata?: Record<string, string> | undefined;
    input?: Record<string, any> | undefined;
}>;
/**
 * Type representing the input for creating a Cron.
 */
export type CreateCronInput = z.infer<typeof CreateCronTriggerSchema>;
/**
 * Client for managing Cron Triggers.
 */
export declare class CronClient {
    private readonly tenantId;
    private readonly config;
    private readonly api;
    private readonly adminClient;
    private logger;
    /**
     * Initializes a new instance of CronClient.
     * @param tenantId - The tenant identifier.
     * @param config - Client configuration settings.
     * @param api - API instance for REST interactions.
     * @param adminClient - Admin client for administrative operations.
     */
    constructor(tenantId: string, config: ClientConfig, api: Api, adminClient: AdminClient);
    /**
     * Retrieves the Cron ID from a CronWorkflows object or a string.
     * @param cron - The CronWorkflows object or Cron ID as a string.
     * @returns The Cron ID as a string.
     */
    private getCronId;
    /**
     * Creates a new Cron workflow.
     * @param workflow - The workflow identifier or Workflow object.
     * @param cron - The input data for creating the Cron Trigger.
     * @returns A promise that resolves to the created CronWorkflows object.
     * @throws Will throw an error if the input is invalid or the API call fails.
     */
    create(workflow: string | Workflow, cron: CreateCronInput): Promise<CronWorkflows>;
    /**
     * Deletes an existing Cron Trigger.
     * @param cron - The Cron Trigger ID as a string or CronWorkflows object.
     * @returns A promise that resolves when the Cron Trigger is deleted.
     */
    delete(cron: string | CronWorkflows): Promise<void>;
    /**
     * Lists all Cron Triggers based on the provided query parameters.
     * @param query - Query parameters for listing Cron Triggers.
     * @returns A promise that resolves to a CronWorkflowsList object.
     */
    list(query: Parameters<typeof this.api.cronWorkflowList>[1]): Promise<CronWorkflowsList>;
    /**
     * Retrieves a specific Cron Trigger by its ID.
     * @param cron - The Cron Trigger ID as a string or CronWorkflows object.
     * @returns A promise that resolves to the CronWorkflows object.
     */
    get(cron: string | CronWorkflows): Promise<CronWorkflows>;
}
