import { ClientConfig, InternalHatchetClient, HatchetClientOptions } from '../../clients/hatchet-client';
import { AxiosRequestConfig } from 'axios';
import WorkflowRunRef from '../../util/workflow-run-ref';
import { Workflow as V0Workflow } from '../../workflow';
import { JsonObject } from '../../step';
import { CreateWorkflowOpts, RunOpts, WorkflowDeclaration } from '../workflow';
import { IHatchetClient } from './client.interface';
import { CreateWorkerOpts, Worker } from './worker';
/**
 * HatchetV1 implements the main client interface for interacting with the Hatchet workflow engine.
 * It provides methods for creating and executing workflows, as well as managing workers.
 */
export declare class HatchetClient implements IHatchetClient {
    /** The underlying v0 client instance */
    v0: InternalHatchetClient;
    /** The tenant ID for the Hatchet client */
    get tenantId(): string;
    /**
     * Creates a new Hatchet client instance.
     * @param config - Optional configuration for the client
     * @param options - Optional client options
     * @param axiosConfig - Optional Axios configuration for HTTP requests
     */
    constructor(config?: Partial<ClientConfig>, options?: HatchetClientOptions, axiosConfig?: AxiosRequestConfig);
    /**
     * Static factory method to create a new Hatchet client instance.
     * @param config - Optional configuration for the client
     * @param options - Optional client options
     * @param axiosConfig - Optional Axios configuration for HTTP requests
     * @returns A new Hatchet client instance
     */
    static init(config?: Partial<ClientConfig>, options?: HatchetClientOptions, axiosConfig?: AxiosRequestConfig): HatchetClient;
    /**
     * Creates a new workflow definition.
     * @template T - The input type for the workflow
     * @template K - The return type of the workflow
     * @param options - Configuration options for creating the workflow
     * @returns A new Workflow instance
     * @note It is possible to create an orphaned workflow if no client is available using @hatchet/client CreateWorkflow
     */
    workflow<T extends JsonObject = any, K extends JsonObject = any>(options: CreateWorkflowOpts): WorkflowDeclaration<T, K>;
    /**
     * Triggers a workflow run without waiting for completion.
     * @template T - The input type for the workflow
     * @template K - The return type of the workflow
     * @param workflow - The workflow to run, either as a Workflow instance or workflow name
     * @param input - The input data for the workflow
     * @param options - Configuration options for the workflow run
     * @returns A WorkflowRunRef containing the run ID and methods to interact with the run
     */
    enqueue<T extends JsonObject = any, K extends JsonObject = any>(workflow: WorkflowDeclaration<T, K> | string | V0Workflow, input: T, options: RunOpts): WorkflowRunRef<K>;
    /**
     * Triggers a workflow run and waits for the result.
     * @template T - The input type for the workflow
     * @template K - The return type of the workflow
     * @param workflow - The workflow to run, either as a Workflow instance or workflow name
     * @param input - The input data for the workflow
     * @param options - Configuration options for the workflow run
     * @returns A promise that resolves with the workflow result
     */
    run<T extends JsonObject = any, K extends JsonObject = any>(workflow: WorkflowDeclaration<T, K> | string | V0Workflow, input: T, options?: RunOpts): Promise<K>;
    get cron(): import("../../clients/hatchet-client/features/cron-client").CronClient;
    get schedule(): import("../../clients/hatchet-client/features/schedule-client").ScheduleClient;
    get event(): import("../../clients/event/event-client").EventClient;
    get api(): import("../..").Api<unknown>;
    /**
     * @deprecated use workflow.run or client.run instead
     */
    get admin(): import("../..").AdminClient;
    /**
     * Creates a new worker instance for processing workflow tasks.
     * @param options - Configuration options for creating the worker
     * @returns A promise that resolves with a new HatchetWorker instance
     */
    worker(name: string, options?: CreateWorkerOpts | number): Promise<Worker>;
    /**
     * Register a webhook with the worker
     * @param workflows - The workflows to register on the webhooks
     * @returns A promise that resolves when the webhook is registered
     */
    webhooks(workflows: WorkflowDeclaration<any, any>[] | V0Workflow[]): import("../../clients/worker/handler").WebhookHandler;
}
