"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HatchetClient = void 0;
const hatchet_client_1 = require("../../clients/hatchet-client");
const workflow_1 = require("../workflow");
const worker_1 = require("./worker");
/**
 * HatchetV1 implements the main client interface for interacting with the Hatchet workflow engine.
 * It provides methods for creating and executing workflows, as well as managing workers.
 */
class HatchetClient {
    /** The tenant ID for the Hatchet client */
    get tenantId() {
        return this.v0.tenantId;
    }
    /**
     * Creates a new Hatchet client instance.
     * @param config - Optional configuration for the client
     * @param options - Optional client options
     * @param axiosConfig - Optional Axios configuration for HTTP requests
     */
    constructor(config, options, axiosConfig) {
        this.v0 = new hatchet_client_1.InternalHatchetClient(config, options, axiosConfig);
    }
    /**
     * Static factory method to create a new Hatchet client instance.
     * @param config - Optional configuration for the client
     * @param options - Optional client options
     * @param axiosConfig - Optional Axios configuration for HTTP requests
     * @returns A new Hatchet client instance
     */
    static init(config, options, axiosConfig) {
        return new HatchetClient(config, options, axiosConfig);
    }
    /**
     * Creates a new workflow definition.
     * @template T - The input type for the workflow
     * @template K - The return type of the workflow
     * @param options - Configuration options for creating the workflow
     * @returns A new Workflow instance
     * @note It is possible to create an orphaned workflow if no client is available using @hatchet/client CreateWorkflow
     */
    workflow(options) {
        return (0, workflow_1.CreateWorkflow)(options, this);
    }
    /**
     * Triggers a workflow run without waiting for completion.
     * @template T - The input type for the workflow
     * @template K - The return type of the workflow
     * @param workflow - The workflow to run, either as a Workflow instance or workflow name
     * @param input - The input data for the workflow
     * @param options - Configuration options for the workflow run
     * @returns A WorkflowRunRef containing the run ID and methods to interact with the run
     */
    enqueue(workflow, input, options) {
        let name;
        if (typeof workflow === 'string') {
            name = workflow;
        }
        else if ('id' in workflow) {
            name = workflow.id;
        }
        else {
            throw new Error('unable to identify workflow');
        }
        return this.v0.admin.runWorkflow(name, input, options);
    }
    /**
     * Triggers a workflow run and waits for the result.
     * @template T - The input type for the workflow
     * @template K - The return type of the workflow
     * @param workflow - The workflow to run, either as a Workflow instance or workflow name
     * @param input - The input data for the workflow
     * @param options - Configuration options for the workflow run
     * @returns A promise that resolves with the workflow result
     */
    run(workflow_2, input_1) {
        return __awaiter(this, arguments, void 0, function* (workflow, input, options = {}) {
            const run = this.enqueue(workflow, input, options);
            return run.result();
        });
    }
    get cron() {
        return this.v0.cron;
    }
    get schedule() {
        return this.v0.schedule;
    }
    get event() {
        return this.v0.event;
    }
    get api() {
        return this.v0.api;
    }
    /**
     * @deprecated use workflow.run or client.run instead
     */
    get admin() {
        return this.v0.admin;
    }
    /**
     * Creates a new worker instance for processing workflow tasks.
     * @param options - Configuration options for creating the worker
     * @returns A promise that resolves with a new HatchetWorker instance
     */
    worker(name, options) {
        let opts = {};
        if (typeof options === 'number') {
            opts = { slots: options };
        }
        else {
            opts = options || {};
        }
        return worker_1.Worker.create(this.v0, name, opts);
    }
    /**
     * Register a webhook with the worker
     * @param workflows - The workflows to register on the webhooks
     * @returns A promise that resolves when the webhook is registered
     */
    webhooks(workflows) {
        return this.v0.webhooks(workflows);
    }
}
exports.HatchetClient = HatchetClient;
