import { WorkerLabels } from '../../clients/dispatcher/dispatcher-client';
import { InternalHatchetClient } from '../../clients/hatchet-client';
import { V0Worker } from '../../clients/worker';
import { Workflow as V0Workflow } from '../../workflow';
import { WebhookWorkerCreateRequest } from '../../clients/rest/generated/data-contracts';
import { WorkflowDeclaration } from '../workflow';
/**
 * Options for creating a new hatchet worker
 * @interface CreateWorkerOpts
 */
export interface CreateWorkerOpts {
    /** Maximum number of concurrent runs on this worker */
    slots?: number;
    /** Array of workflows to register */
    workflows?: WorkflowDeclaration<any, any>[] | V0Workflow[];
    /** Worker labels for affinity-based assignment */
    labels?: WorkerLabels;
    /** Whether to handle kill signals */
    handleKill?: boolean;
    /** @deprecated Use slots instead */
    maxRuns?: number;
}
/**
 * HatchetWorker class for workflow execution runtime
 */
export declare class Worker {
    /** Internal reference to the underlying V0 worker implementation */
    v0: V0Worker;
    /**
     * Creates a new HatchetWorker instance
     * @param v0 - The V0 worker implementation
     */
    constructor(v0: V0Worker);
    /**
     * Creates and initializes a new HatchetWorker
     * @param v0 - The HatchetClient instance
     * @param options - Worker creation options
     * @returns A new HatchetWorker instance
     */
    static create(v0: InternalHatchetClient, name: string, options: CreateWorkerOpts): Promise<Worker>;
    /**
     * Registers workflows with the worker
     * @param workflows - Array of workflows to register
     * @returns Array of registered workflow promises
     */
    registerWorkflows(workflows?: Array<WorkflowDeclaration<any, any> | V0Workflow>): Promise<void[]>;
    /**
     * Registers a single workflow with the worker
     * @param workflow - The workflow to register
     * @returns A promise that resolves when the workflow is registered
     * @deprecated use registerWorkflows instead
     */
    registerWorkflow(workflow: WorkflowDeclaration<any, any> | V0Workflow): Promise<void[]>;
    /**
     * Starts the worker
     * @returns Promise that resolves when the worker is stopped or killed
     */
    start(): Promise<void>;
    /**
     * Stops the worker
     * @returns Promise that resolves when the worker stops
     */
    stop(): Promise<void>;
    /**
     * Updates or inserts worker labels
     * @param labels - Worker labels to update
     * @returns Promise that resolves when labels are updated
     */
    upsertLabels(labels: WorkerLabels): Promise<WorkerLabels>;
    /**
     * Get the labels for the worker
     * @returns The labels for the worker
     */
    getLabels(): WorkerLabels;
    /**
     * Register a webhook with the worker
     * @param webhook - The webhook to register
     * @returns A promise that resolves when the webhook is registered
     */
    registerWebhook(webhook: WebhookWorkerCreateRequest): Promise<import("axios").AxiosResponse<import("../../clients/rest/generated/data-contracts").WebhookWorkerCreated, any>>;
}
export declare function toV0Workflow(wf: WorkflowDeclaration<any, any> | V0Workflow): V0Workflow;
