"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Worker = void 0;
exports.toV0Workflow = toV0Workflow;
const workflow_1 = require("../workflow");
/**
 * HatchetWorker class for workflow execution runtime
 */
class Worker {
    /**
     * Creates a new HatchetWorker instance
     * @param v0 - The V0 worker implementation
     */
    constructor(v0) {
        this.v0 = v0;
    }
    /**
     * Creates and initializes a new HatchetWorker
     * @param v0 - The HatchetClient instance
     * @param options - Worker creation options
     * @returns A new HatchetWorker instance
     */
    static create(v0, name, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const v0worker = yield v0.worker(name, Object.assign(Object.assign({}, options), { maxRuns: options.slots || options.maxRuns }));
            const worker = new Worker(v0worker);
            yield worker.registerWorkflows(options.workflows);
            return worker;
        });
    }
    /**
     * Registers workflows with the worker
     * @param workflows - Array of workflows to register
     * @returns Array of registered workflow promises
     */
    registerWorkflows(workflows) {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.all((workflows === null || workflows === void 0 ? void 0 : workflows.map((wf) => {
                return this.v0.registerWorkflow(toV0Workflow(wf));
            })) || []);
        });
    }
    /**
     * Registers a single workflow with the worker
     * @param workflow - The workflow to register
     * @returns A promise that resolves when the workflow is registered
     * @deprecated use registerWorkflows instead
     */
    registerWorkflow(workflow) {
        return this.registerWorkflows([workflow]);
    }
    /**
     * Starts the worker
     * @returns Promise that resolves when the worker is stopped or killed
     */
    start() {
        return this.v0.start();
    }
    /**
     * Stops the worker
     * @returns Promise that resolves when the worker stops
     */
    stop() {
        return this.v0.stop();
    }
    /**
     * Updates or inserts worker labels
     * @param labels - Worker labels to update
     * @returns Promise that resolves when labels are updated
     */
    upsertLabels(labels) {
        return this.v0.upsertLabels(labels);
    }
    /**
     * Get the labels for the worker
     * @returns The labels for the worker
     */
    getLabels() {
        return this.v0.labels;
    }
    /**
     * Register a webhook with the worker
     * @param webhook - The webhook to register
     * @returns A promise that resolves when the webhook is registered
     */
    registerWebhook(webhook) {
        return this.v0.registerWebhook(webhook);
    }
}
exports.Worker = Worker;
function toV0Workflow(wf) {
    if (wf instanceof workflow_1.WorkflowDeclaration) {
        const { definition } = wf;
        return {
            id: definition.name,
            description: definition.description || '',
            version: definition.version || '',
            sticky: definition.sticky,
            scheduleTimeout: definition.scheduleTimeout,
            on: definition.on,
            concurrency: definition.concurrency,
            onFailure: definition.onFailure && {
                name: 'on-failure',
                run: (ctx) => definition.onFailure(ctx),
            },
            steps: definition.tasks.map((task) => {
                var _a;
                return ({
                    name: task.name,
                    parents: (_a = task.parents) === null || _a === void 0 ? void 0 : _a.map((p) => p.name),
                    run: (ctx) => task.fn(ctx.workflowInput(), ctx),
                    timeout: task.timeout,
                    retries: task.retries,
                    rate_limits: task.rateLimits,
                    worker_labels: task.workerLabels,
                    backoff: task.backoff,
                });
            }),
        };
    }
    return wf;
}
