import WorkflowRunRef from '../util/workflow-run-ref';
import { Context, JsonObject } from '../step';
import { CronWorkflows, ScheduledWorkflows } from '../clients/rest/generated/data-contracts';
import { Workflow as WorkflowV0 } from '../workflow';
import { IHatchetClient } from './client/client.interface';
import { CreateTaskOpts } from './task';
/**
 * Additional metadata that can be attached to a workflow run.
 */
type AdditionalMetadata = Record<string, string>;
/**
 * Options for running a workflow.
 */
export type RunOpts = {
    /**
     * Additional metadata to attach to the workflow run.
     */
    additionalMetadata?: AdditionalMetadata;
};
/**
 * Extracts a property from an object type based on task name, or falls back to inferred type
 */
type TaskOutputType<K, TaskName extends string, InferredType> = TaskName extends keyof K ? K[TaskName] : InferredType;
/**
 * Options for creating a new workflow.
 */
export type CreateWorkflowOpts = {
    /**
     * The name of the workflow.
     */
    name: WorkflowV0['id'];
    /**
     * (optional) description of the workflow.
     */
    description?: WorkflowV0['description'];
    /**
     * (optional) version of the workflow.
     */
    version?: WorkflowV0['version'];
    /**
     * (optional) sticky strategy for the workflow.
     */
    sticky?: WorkflowV0['sticky'];
    /**
     * (optional) schedule timeout for the workflow.
     */
    scheduleTimeout?: WorkflowV0['scheduleTimeout'];
    /**
     * (optional) on config for the workflow.
     */
    on?: WorkflowV0['on'];
    concurrency?: WorkflowV0['concurrency'];
    /**
     * (optional) onFailure handler for the workflow.
     * Invoked when any task in the workflow fails.
     * @param ctx The context of the workflow.
     */
    onFailure?: (ctx: Context<any>) => void;
};
/**
 * Internal definition of a workflow and its tasks.
 */
type WorkflowDefinition = CreateWorkflowOpts & {
    /**
     * The tasks that make up this workflow.
     */
    tasks: CreateTaskOpts<any, any>[];
};
/**
 * Represents a workflow that can be executed by Hatchet.
 * @template T The input type for the workflow.
 * @template K The return type of the workflow.
 */
export declare class WorkflowDeclaration<T extends JsonObject, K extends JsonObject> {
    /**
     * The Hatchet client instance used to execute the workflow.
     */
    client: IHatchetClient | undefined;
    /**
     * The internal workflow definition.
     */
    definition: WorkflowDefinition;
    /**
     * Creates a new workflow instance.
     * @param options The options for creating the workflow.
     * @param client Optional Hatchet client instance.
     */
    constructor(options: CreateWorkflowOpts, client?: IHatchetClient);
    /**
     * Triggers a workflow run without waiting for completion.
     * @param input The input data for the workflow.
     * @param options Optional configuration for this workflow run.
     * @returns A WorkflowRunRef containing the run ID and methods to get results and interact with the run.
     * @throws Error if the workflow is not bound to a Hatchet client.
     */
    enqueue(input: T, options?: RunOpts): WorkflowRunRef<K>;
    /**
     * Executes the workflow with the given input and awaits the results.
     * @param input The input data for the workflow.
     * @param options Optional configuration for this workflow run.
     * @returns A promise that resolves with the workflow result.
     * @throws Error if the workflow is not bound to a Hatchet client.
     */
    run(input: T, options?: RunOpts): Promise<K>;
    run(input: T[], options?: RunOpts): Promise<K[]>;
    /**
     * Schedules a workflow to run at a specific date and time in the future.
     * @param enqueueAt The date when the workflow should be triggered.
     * @param input The input data for the workflow.
     * @param options Optional configuration for this workflow run.
     * @returns A promise that resolves with the scheduled workflow details.
     * @throws Error if the workflow is not bound to a Hatchet client.
     */
    schedule(enqueueAt: Date, input: T, options?: RunOpts): Promise<ScheduledWorkflows>;
    /**
     * Schedules a workflow to run after a specified delay.
     * @param duration The delay in seconds before the workflow should run.
     * @param input The input data for the workflow.
     * @param options Optional configuration for this workflow run.
     * @returns A promise that resolves with the scheduled workflow details.
     * @throws Error if the workflow is not bound to a Hatchet client.
     */
    delay(duration: number, input: T, options?: RunOpts): Promise<ScheduledWorkflows>;
    /**
     * Creates a cron schedule for the workflow.
     * @param name The name of the cron schedule.
     * @param expression The cron expression defining the schedule.
     * @param input The input data for the workflow.
     * @param options Optional configuration for this workflow run.
     * @returns A promise that resolves with the cron workflow details.
     * @throws Error if the workflow is not bound to a Hatchet client.
     */
    cron(name: string, expression: string, input: T, options?: RunOpts): Promise<CronWorkflows>;
    /**
     * Adds a task to the workflow.
     * The return type will be either the property on K that corresponds to the task name,
     * or if there is no matching property, the inferred return type of the function.
     * @template Name The literal string name of the task.
     * @template L The inferred return type of the task function.
     * @param options The task configuration options.
     * @returns The task options that were added.
     */
    task<Name extends string, L>(options: Omit<CreateTaskOpts<T, TaskOutputType<K, Name, L>>, 'fn'> & {
        name: Name;
        fn: (input: T, ctx: Context<T>) => TaskOutputType<K, Name, L> | Promise<TaskOutputType<K, Name, L>>;
    }): CreateTaskOpts<T, TaskOutputType<K, Name, L>>;
    get id(): string;
}
/**
 * Creates a new workflow instance.
 * @template T The input type for the workflow.
 * @template K The return type of the workflow.
 * @param options The options for creating the workflow.
 * @param client Optional Hatchet client instance.
 * @returns A new Workflow instance.
 */
export declare function CreateWorkflow<T extends JsonObject = any, K extends JsonObject = any>(options: CreateWorkflowOpts, client?: IHatchetClient): WorkflowDeclaration<T, K>;
export {};
